;yi bullet bill
;by smkdan

;this reqiores dsx.asm in the patch directory to be applied to your ROM

;GRAPHICS ARE IN DCX.BIN

;C2: frame index
;1570: frame counter for fire timer
;1602: firing?
;1528: frame indexer, applied to C2

;set this to the inserted bullet bill sprite
SPRITE_GEN = $0E

;time between fires
FIRE_INTERVAL = $C0

;sound to 1DFC on fire
FIRE_SOUND = $09

;frames indexed by C2, 28 total
FRAMEINDEXES 	dcb	$01,$02,$03,$04,$05,$06,$07,$08	;load frames
		dcb	$09,$0A,$0B

		dcb	$0C,$0D,$0E,$0F,$10,$11,$11,$10	;fired frames
		dcb	$0F,$0E,$0D,$0C,$0D,$0E,$0F,$10
		dcb	$10

YDISPSTBL	dcw	$FFF8,$FFF8,$FFF7,$FFF7,$FFF6,$FFF6,$FFF5,$FFF5	;load frames
		dcw	$FFF5,$FFF5,$FFF4

		dcw	$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8	;fired frames
		dcw	$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8,$FFF8
		dcw	$FFF8

PROPRAM = $04
TEMP = $09

SPEED	dcb $F8,$08

dcb "INIT"
	JSR SUB_HORZ_POS	;face mario
	TYA
	EOR #$01
	STA $157C,x	
	RTL

dcb "MAIN"
	PHB
	PHK
	PLB
	JSR Run
	PLB
	RTL


Run
	JSR SUB_OFF_SCREEN_X0
	JSR GET_DRAW_INFO
	JSR GFX			;draw sprite

	LDA $14C8,x
	CMP #$08          	 
	BNE Return           
	LDA $9D			;locked sprites?
	BNE Return
	LDA $15D0,x		;yoshi eating?
	BNE Return

;check already firing
	LDA $1602,x		;only advance counter when it isn't firing
	BNE FiringAnimation

;counter
	INC $1570,x		;advance fire interval...
	LDA $1570,x		;check firing interval
	CMP #FIRE_INTERVAL
	BNE DontFire

;firing
	LDA #$01		;set firing status true
	STA $1602,x

	LDA $1602,x		;check firing status
	BEQ DontFire		;loading would be the wierd animation it does

FiringAnimation
;two speeds depending on index range
	LDA $1528,x		;current indexer
	CMP #$0B		;11
	BCS FastTick

;slowtick
	LDA $14			;time from fc
	AND #$03		;every 4th frame
	BNE DontFire
	BRA SkipTick
FastTick
	LDA $14
	AND #$01		;every 2nd frame
	BNE DontFire

SkipTick
	INC $1528,x		;advance frame
	LDY $1528,x		;apply to C2
	LDA FRAMEINDEXES,y
	STA $C2,x		;apply to GFX frame index
	LDA $1528,x		;test indexer
	CMP #$0B		;frame 11 = shoot
	BEQ Fire		;if not yet reached, just keep advancing
	CMP #$1C		;frame 24 = done
	BEQ EndAnimation
	BRA DontFire		;bottom out

Fire
	JSR GenerateC		;generate bullet bill
	LDA #FIRE_SOUND		;play firing sound
	STA $1DFC
	BRA DontFire		;skip past stuff
		
EndAnimation
	STZ $1602,x		;no longer firing
	STZ $1528,x		;reset frame index
	STZ $1570,x		;reset firing interval
	STZ $C2,x		;reset frame

DontFire
;currently just store frame 0 (idling)
	LDA $1528,x		;load frame index to use as YDISPS index
	ASL A			;16bits per entry
	TAY			;Y index

	REP #$20		;16bit A
	LDA YDISPSTBL,y		;load entry
	STA TEMP		;ram solidity uses
	SEP #$20		;8bit A

	JSR SOLIDITY		;handle being solid

	JSL $01802A		;update speed, only used for falling

Return
	RTS        
			
;=====

STANDTILE = $80
STANDPROP = $03

FRAMES	dcb $00,$01,$02,$03
	dcb $10,$11,$12,$13
	dcb $20,$21,$22,$23
	dcb $30,$31,$32,$33
	dcb $40,$41,$42,$43

XDISP	dcb $00,$10,$00,$10
YDISP	dcb $02,$02,$12,$12
TILEMAP	dcb $00,$02,$20,$22	;test map

EORS	dcb $0F,$00

ReturnGl
	RTS
GFX
;first, display the stand tile unconditionally

	LDA $00
	STA $0300,y

	LDA $01
	STA $0301,y

	LDA #STANDTILE
	STA $0302,y

	LDA #STANDPROP
	STA $0303,y

	INY
	INY
	INY
	INY		
	
;start displaying dynamic frames
	LDA $C2,x	;load frame index

	PHX		;calc frame
	TAX
	LDA FRAMES,x	;load a temp frame
	PLX

	JSR GETSLOT	;A = frame.  grab a slot if avaliable.  On return, the starting tile number will be in A, 00 signals failure
	BEQ ReturnGl	;return on none avaliable
	STA TEMP	;store to scratch

	LDA $15F6,x	;properties...
	STA PROPRAM

	LDX #$00	;zero index

OAM_LOOP
	LDA $00
	CLC
	ADC XDISP,x
	SEC
	SBC #$08	;-8px
	STA $0300,y

	LDA $01
	CLC
	ADC YDISP,x
	SEC
	SBC #$12	;-18px
	STA $0301,y

	LDA TILEMAP,x
	CLC
	ADC TEMP	;add calculated tile
	STA $0302,y

	LDA PROPRAM
	ORA $64
	STA $0303,y

	INY
	INY
	INY
	INY
	INX
	CPX #$04
	BNE OAM_LOOP

	LDX $15E9	;restore sprite index 

        LDY #$02
        LDA #$04               ;5 tiles
        JSL $01B7B3

	RTS
ReturnG
	LDX $15E9	;restore sprite index 
	RTS

;generate custom sprite
;======================

GENPOS	dcw $0010,$FFF0

GenerateC	
	JSL $02A9E4		;grab free sprite slot
	BMI ReturnGC		;return if none left...

	LDA #$01
	STA $14C8,y		;normal status, run init routine?

	LDA #SPRITE_GEN		;load sprite to generate...
	PHX			;only xreg can be used with long absolute indexed
	TYX			;Y->X
	STA $7FAB9E,X
	TXY			;X->Y
	PLX			;and restore  

	LDA $E4,x		;store 16bits xposition at $00
	STA $00
	LDA $14E0,x
	STA $01

	PHY
	JSR SUB_HORZ_POS	;genpos depending on size
	TYA
	ASL A			;16bits
	TAY
	LDA GENPOS,y
	CLC
	ADC $00
	PLY

	STA $E4,y		;store ypositions
	XBA
	STA $14E0,y

	LDA $D8,x
	SEC
	SBC #$0F		;-15
	STA $D8,y		;Ypos
	LDA $14D4,x
	SBC #$00		;transfer carry
	STA $14D4,y		;same screen

	TYX			;new sprite slot into X
	JSL $07F7D2		;create
	JSL $0187A7
        LDA #$88		;set as custom sprite
        STA $7FAB10,X

	LDX $15E9		;restore sprite index

	PHY
	JSR SUB_HORZ_POS	;direction based on mario side
	TYA
	PLY
	STA $157C,y		;set direction
		
ReturnGC
	RTS  

;my solidity routine, beefier than it should be i think
;slightly hacked to replace YDISPS with value in RAM

XDISPS = $FFF2		;xpos relative to spritex
;YDISPS = $FFF8		;ram substitute
WIDTH = $000E		;width of interaction field
HEIGHT = $0018

;TEMP: dynamic YDISP

SOLIDITY
	PHP		;preserve status
	SEP #$30	;8bit AXY
	STZ $00		;reset scratch ram used here
	STZ $01
	STZ $02

;optional yoshi displacement
	LDY #$E0	;yoshiless displacement
	LDA $187A	;yoshi flag
	BEQ NoYoshiB
	LDY #$D0	;yoshi displacement
NoYoshiB
	TYA		;Y->A
	STA $04
	LDA #$FF	;signed
	STA $05

;test left edge
	LDA $14E0,x	;xhigh
	XBA
	LDA $E4,x	;xlow
	REP #$20	;16bit A
	CLC
	ADC.w #XDISPS	;add displacement word
	CMP $94		;compare mario x
	SEP #$20	;8bit A again
	BCS HorzOut	;if Mario's X is smaller, then mario is to the left of sprite

;within from left, test right
	LDA $14E0,x	;load x positions again
	XBA
	LDA $E4,x
	REP #$20	;16bit A
	CLC
	ADC.w #WIDTH	;add width, test right
	STA $00		;scratch
	LDA $94		;load mario X
	CMP $00		;compare operated value
	SEP #$20	;8bit A again
	BCS HorzOut

	LDA #$01	;within horizontal range, set bit0 of $02
	TSB $02

HorzOut			;skip label for horizontal range, now test vertical range
;test Ytop A
	LDA $14D4,x	;yhigh
	XBA
	LDA $D8,x	;ylow
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add displacement word
	ADC TEMP
	CLC
	ADC $04
	CMP $96		;compare mario y
	SEP #$20	;8bit A again
	BCS VertTopOut	;if Mario's Y is smaller, then mario is ? of sprite

;Ytop B
	LDA $14D4,x	;load x positions again
	XBA
	LDA $D8,x
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add width, test right
	ADC TEMP
	CLC
	ADC.w #$0006	;6px 'bufferzone'
	CLC
	ADC $04
	STA $00		;scratch
	LDA $96		;load mario X
	CMP $00		;compare operated value
	SEP #$20	;8bit A again
	BCS VertTopOut

	LDA #$02	;within vertical top range
	TSB $02
	JMP SkipVerts

VertTopOut		;not in YTop, try Y center
;test Ycent A
	LDA $14D4,x	;yhigh
	XBA
	LDA $D8,x	;ylow
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add displacement word
	ADC TEMP
	CLC
	ADC.w #$0006	;add 6 px
	CLC
	ADC $04
	CMP $96		;compare mario y
	SEP #$20	;8bit A again
	BCS VertCentOut	;if Mario's Y is smaller, then mario is ? of sprite

;Ycent B
	LDA $14D4,x	;load x positions again
	XBA
	LDA $D8,x
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add width, test right
	ADC TEMP
	CLC
	ADC.w #HEIGHT	;add height
	SEC
	SBC.w #$0006	;6px 'buffer zone' too
	CLC
	ADC.w #$0010	;16px
	CLC
	ADC $04
	STA $00		;scratch
	LDA $96		;load mario X
	CMP $00		;compare operated value
	SEP #$20	;8bit A again
	BCS VertCentOut

	LDA #$04	;within vertical cent range
	TSB $02
	BRA SkipVerts

VertCentOut		;not in Ycent, try Y bottom
	LDA $14D4,x	;yhigh
	XBA
	LDA $D8,x	;ylow
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add displacement word
	ADC TEMP
	CLC
	ADC.w #HEIGHT
	CLC
	ADC.w #$0010	;add 16px, since it's relative to mario's head instead of his feet here
	SEC
	SBC.w #$0006	;sub 6px buffer
	CLC
	ADC $04
	CMP $96		;compare mario y
	SEP #$20	;8bit A again
	BCS SkipVerts

;Ybot B
	LDA $14D4,x	;load x positions again
	XBA
	LDA $D8,x
	REP #$20	;16bit A
	CLC
	;ADC.w #YDISPS	;add width, test right
	ADC TEMP
	CLC
	ADC.w #HEIGHT	;add height
	CLC
	ADC.w #$0016	;add 16px for marios head again
	CLC
	ADC $04
	STA $00		;scratch
	LDA $96		;load mario X
	CMP $00		;compare operated value
	SEP #$20	;8bit A again
	BCS SkipVerts

	LDA #$08	;within vertical bottom range
	TSB $02

SkipVerts
;rising mario

;placement on top of sprite
	LDA $02		;test 2 bits for X range and Y top
	AND #$03	;low 2 bits
	CMP #$03
	BEQ HandleStand	;handle mario standingon platform

	LDA $02
	AND #$05	;test xrange and ycent
	CMP #$05
	BEQ HandleCent

	LDA $02
	AND #$09	;test xbottom and ybottom
	CMP #$09
	BEQ HandleBumpHead

	JMP ReturnSLD	;bottom out

HandleCent
	JSR SUB_HORZ_POS	;get side of sprite mario is on
	CPY #$00	;compare result
	BEQ MarioLeftRight

;Marioleftright
	LDA $7B		;load xspd
	BMI NoClipXY	;don't clip if going in direction that isn't blocked
	STZ $7B		;clip if required
	JMP ReturnSLD	;ret

MarioLeftRight
	LDA $7B		;load xspd
	BPL NoClipXY
	STZ $7B	
	JMP ReturnSLD	;ret

NoClipXY
	JMP ReturnSLD	;return

HandleBumpHead
	LDA $7D		;if rising, set falling speed
	BPL DontClipYS
	LDA #$08	;set falling speed
	STA $7D
DontClipYS
	JMP ReturnSLD	;return

HandleStand
	LDA $7D		;mario Y
	BPL FallingMario

	;handle bumping from the underside if needed
	BRA ReturnSLD	;TEMP: return
	
FallingMario
	SEP #$20	;8bit A
	LDA #$01	;set riding sprite
	STA $1471
	STZ $7D		;no Yspd

	LDY #$E0	;displacement no yoshi
	LDA $187A	;yoshi flag
	BEQ NoYoshiA
	LDY #$D0	;displacement yoshi
NoYoshiA
	INY
	TYA		;chosen displacement to A
	CLC
	ADC $D8,x	;sprite y lo
	STA $96		;mario y lo
	LDA $14D4,x	;sprite y hi
	ADC #$FF	;inverse carry
	STA $97

;now add ydisps...
	REP #$20	;16bit A
	LDA $96
	CLC
	;ADC.w #YDISPS
	ADC TEMP
	STA $96		;modded ypos

ReturnSLD
	PLP		;restore status
	RTS

;2kb buffer is at 7F:0B44.  It is copied in one chunk to VRAM on NMI.
;variables for following routine
;based off smwedit's code
SLOTPTR = $0660		;16bit pointer for source GFX
SLOTBANK = $0662	;bank
SLOTDEST = $0663	;VRAM address
SLOTSUSED = $06FE	;how many slots have been used

MAXSLOTS = $04		;maximum selected slots

SLOTS dcb $CC,$C8,$C4,$C0	;avaliable slots.  Any more transfers and it's overflowing by a dangerous amount.

GETSLOT
	PHY		;preserve OAM index
	PHA		;preserve frame
	LDA SLOTSUSED	;test if slotsused == maximum allowed
	CMP #MAXSLOTS
	BEQ NoneFree

	PLA		;pop frame
	REP #$20	;16bit A
	AND.w #$00FF	;wipe high
	XBA		;<< 8
	LSR A		;>> 1 = << 7
	STA TEMP	;back to scratch
	PEA SPRITEGFX	;push 16bit address, since i'm not sure how else i'm going to get the address of the label itself in TRASM
	PLA		;pull 16bit address
	CLC
	ADC TEMP	;add frame offset	
	STA SLOTPTR	;store to pointer to be used at transfer time
	SEP #$20	;8bit store
	PHB		;push db...
	PLA		;pull to A...
	STA SLOTBANK	;store bank to 24bit pointer

	LDY SLOTSUSED	;calculate VRAM address + tile number
	LDA SLOTS,y	;get tile# in VRAM
	PHA		;preserve for eventual pull
	SEC
	SBC #$C0	;starts at C0h, they start at C0 in tilemap
	REP #$20	;16bit math
	AND.w #$00FF	;wipe high byte
	ASL A		;multiply by 32, since 32 bytes/16 words equates to 1 32bytes tile
	ASL A
	ASL A
	ASL A
	ASL A
	CLC
	ADC #$0B44	;add 0B44, base address of buffer	
	STA SLOTDEST	;destination address in the buffer

	JSR DMABUFFER	;ROM -> RAM copy

	SEP #$20	;8bit A	
	INC SLOTSUSED	;one extra slot has been used

	PLA		;return starting tile number
	PLY
	RTS
		
NoneFree
	PLA
	PLY
	LDA #$00	;zero on no free slots
	RTS

;;;;;;;;;;;;;;;;
;Tansfer routine
;;;;;;;;;;;;;;;;

;DMA ROM -> RAM ROUTINE

DMABUFFER
;set destination RAM address
	REP #$20
	LDA SLOTDEST
	STA $2181	;16bit RAM dest
	LDY #$7F
	STY $2183	;set 7F as bank

;common DMA settings
	STZ $4300	;1 reg only
	LDY #$80	;to 2180, RAM write/read
	STY $4301

;first line
	LDA SLOTPTR
	STA $4302	;low 16bits
	LDY SLOTBANK
	STY $4304	;bank
	LDY #$80	;128 bytes
	STY $4305
	LDY #$01
	STY $420B	;transfer

;second line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR	;update source address
	CLC
	ADC #$0200	;512 bytes, next row
	STA SLOTPTR
	STA $4302	;low 16bits
	LDY SLOTBANK
	STY $4304	;bank
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B	;transfer

;third line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR	;update
	CLC
	ADC #$0200
	STA SLOTPTR
	STA $4302
	LDY SLOTBANK
	STY $4304
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B	;transfer

;fourth line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR
	CLC
	ADC #$0200
	STA SLOTPTR
	STA $4302
	LDY SLOTBANK
	STY $4304
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B

	SEP #$20	;8bit A
	RTS		;all done, return

;=================
;BORROWED ROUTINES
;=================

;GET_DRAW_INFO
;==============

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;SUBHORZPOS, disp mario by 8px
;==========

SUB_HORZ_POS       	REP #$20		;16bitA
			LDA $94
			PHA
			CLC
			ADC.w #$0008
			STA $94
			SEP #$20				
		    LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             
			REP #$20
			PLA
			STA $94
			SEP #$20
		     RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;SUBVERTPPOS
;===========

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924

SPRITEGFX
INCBIN sprites\cdx.bin	;included graphics file




